package com.xhpc.card.mapper;

import com.xhpc.card.domain.SearchCardUserQueryCondition;
import com.xhpc.card.pojo.XhpcAppUser;

import java.util.List;

public interface XhpcAppUserMapper {

    int deleteByPrimaryKey(Long appUserId);

    int insert(XhpcAppUser record);

    int insertSelective(XhpcAppUser record);

    XhpcAppUser selectByPrimaryKey(Long appUserId);

    int updateByPrimaryKeySelective(XhpcAppUser record);

    int updateByPrimaryKey(XhpcAppUser record);

    /**
     * 查询记录总数
     *
     * @author WH
     * @date 2022/1/28 16:30
     * @since version-1.0
     */
    Long selectCount(SearchCardUserQueryCondition queryCondition);

    /**
     * 根据条件查询记录
     *
     * @author WH
     * @date 2022/1/28 16:29
     * @since version-1.0
     */
    List<XhpcAppUser> selectByCondition(SearchCardUserQueryCondition queryCondition);

}